@@ -65,7 +65,7 @@
 
 #define VERSION_MAJOR 0
 #define VERSION_MINOR 1
-#define VERSION "0.3"
+#define VERSION "0.5"
 
 #ifdef WIN32
 #undef WITH_THREAD /* Thread support locks up xchat on Win32. */
@@ -327,6 +327,9 @@ Util_BuildList(char *word[])
 static void
 Util_Autoload()
 {
+#ifndef PATH_MAX
+#define PATH_MAX 1024	/* Hurd doesn't define it */
+#endif
 	char oldcwd[PATH_MAX];
 	const char *dir_name;
 	struct dirent *ent;
@@ -476,7 +479,7 @@ Callback_Print(char *word[], void *userdata)
 	 * build our own here. */
 	while (word[listsize] && word[listsize][0])
 		listsize++;
-	word_eol = (char **) g_malloc(sizeof(char*)*listsize+1);
+	word_eol = (char **) g_malloc(sizeof(char*)*(listsize+1));
 	if (word_eol == NULL) {
 		xchat_print(ph, "Not enough memory to alloc word_eol "
 				"for python plugin callback.");
@@ -1114,7 +1117,7 @@ Plugin_New(char *filename, PyMethodDef *xchat_methods, PyObject *xcoobj)
 		fp = fopen(plugin->filename, "r");
 		if (fp == NULL) {
 			xchat_printf(ph, "Can't open file %s: %s\n",
-				     filename, strerror(errno));
+				     plugin->filename, strerror(errno));
 			goto error;
 		}
 
@@ -1409,13 +1412,14 @@ static PyObject *
 Module_xchat_get_prefs(PyObject *self, PyObject *args)
 {
 	PyObject *res;
-	const void *info;
+	const char *info;
+	int integer;
 	char *name;
 	int type;
 	if (!PyArg_ParseTuple(args, "s:get_prefs", &name))
 		return NULL;
 	BEGIN_XCHAT_CALLS(NONE);
-	type = xchat_get_prefs(ph, name, (const char**)&info, (int*)&info);
+	type = xchat_get_prefs(ph, name, &info, &integer);
 	END_XCHAT_CALLS();
 	switch (type) {
 		case 0:
@@ -1427,7 +1431,7 @@ Module_xchat_get_prefs(PyObject *self, PyObject *args)
 			break;
 		case 2:
 		case 3:
-			res = PyInt_FromLong((int)info);
+			res = PyInt_FromLong(integer);
 			break;
 		default:
 			PyErr_Format(PyExc_RuntimeError,
@@ -1668,7 +1672,7 @@ Module_xchat_get_list(PyObject *self, PyObject *args)
 	xchat_list *list;
 	PyObject *l;
 	const char *name;
-	const char **fields;
+	const char *const *fields;
 	int i;
 
 	if (!PyArg_ParseTuple(args, "s:get_list", &name))
@@ -1723,6 +1727,7 @@ Module_xchat_get_list(PyObject *self, PyObject *args)
 						(xchat_context*)sattr);
 					break;
 				}
+			default: /* ignore unknown (newly added?) types */
 				continue;
 			}
 			if (attr == NULL)
@@ -1747,7 +1752,7 @@ static PyObject *
 Module_xchat_get_lists(PyObject *self, PyObject *args)
 {
 	PyObject *l, *o;
-	const char **fields;
+	const char *const *fields;
 	int i;
 	/* This function is thread safe, and returns statically
 	 * allocated data. */
@@ -2017,6 +2022,16 @@ Command_Unload(char *word[], char *word_eol[], void *userdata)
 static int initialized = 0;
 static int reinit_tried = 0;
 
+void
+xchat_plugin_get_info(char **name, char **desc, char **version, void **reserved)
+{
+	*name = "Python";
+	*version = VERSION;
+	*desc = "Python scripting interface";
+   if (reserved)
+      *reserved = NULL;
+}
+
 int
 xchat_plugin_init(xchat_plugin *plugin_handle,
 		  char **plugin_name,
