@@ -909,7 +909,7 @@ xchat_set_context (xchat_plugin *ph, xchat_context *context)
 xchat_context *
 xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 {
-	GSList *slist, *clist;
+	GSList *slist, *clist, *sessions = NULL;
 	server *serv;
 	session *sess;
 	char *netname;
@@ -938,7 +938,16 @@ xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 				if (sess->server == serv)
 				{
 					if (rfc_casecmp (channel, sess->channel) == 0)
-						return sess;
+					{
+						if (sess->server == ph->context->server)
+						{
+							g_slist_free (sessions);
+							return sess;
+						} else
+						{
+							sessions = g_slist_prepend (sessions, sess);
+						}
+					}
 				}
 				clist = clist->next;
 			}
@@ -946,6 +955,14 @@ xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 		slist = slist->next;
 	}
 
+	if (sessions)
+	{
+		sessions = g_slist_reverse (sessions);
+		sess = sessions->data;
+		g_slist_free (sessions);
+		return sess;
+	}
+
 	return NULL;
 }
 
@@ -953,6 +970,23 @@ const char *
 xchat_get_info (xchat_plugin *ph, const char *id)
 {
 	session *sess;
+	guint32 hash = str_hash (id);
+
+	/* do the session independant ones first */
+	switch (hash)
+	{
+	case 0x325acab5:	/* libdirfs */
+		return XCHATLIBDIR;
+
+	case 0x14f51cd8: /* version */
+		return PACKAGE_VERSION;
+
+	case 0xdd9b1abd:	/* xchatdir */
+		return get_xdir_utf8 ();
+
+	case 0xe33f6c4a:	/* xchatdirfs */
+		return get_xdir_fs ();
+	}
 
 	sess = ph->context;
 	if (!is_session (sess))
@@ -961,7 +995,7 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 		return NULL;
 	}
 
-	switch (str_hash (id))
+	switch (hash)
 	{
 	case 0x2de2ee: /* away */
 		if (sess->server->is_away)
@@ -989,9 +1023,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x1c0e99c1: /* inputbox */
 		return fe_get_inputbox_contents (sess);
 
-	case 0x325acab5:	/* libdirfs */
-		return XCHATLIBDIR;
-
 	case 0x6de15a2e:	/* network */
 		return server_get_network (sess->server, FALSE);
 
@@ -1011,9 +1042,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x696cd2f: /* topic */
 		return sess->topic;
 
-	case 0x14f51cd8: /* version */
-		return VERSION;
-
 	case 0x506d600b: /* native win_ptr */
 		return fe_gui_info_ptr (sess, 0);
 
@@ -1025,12 +1053,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 		case 2: return "hidden";
 		}
 		return NULL;
-
-	case 0xdd9b1abd:	/* xchatdir */
-		return get_xdir_utf8 ();
-
-	case 0xe33f6c4a:	/* xchatdirfs */
-		return get_xdir_fs ();
 	}
 
 	return NULL;
@@ -1042,12 +1064,17 @@ xchat_get_prefs (xchat_plugin *ph, const char *name, const char **string, int *i
 	int i = 0;
 
 	/* some special run-time info (not really prefs, but may aswell throw it in here) */
-	if (!strcmp (name, "state_cursor"))
+	switch (str_hash (name))
 	{
-		*integer = fe_get_inputbox_cursor (ph->context);
-		return 2;
-	}
+		case 0xf82136c4: /* state_cursor */
+			*integer = fe_get_inputbox_cursor (ph->context);
+			return 2;
 
+		case 0xd1b: /* id */
+			*integer = ph->context->server->id;
+			return 2;
+	}
+	
 	do
 	{
 		if (!strcasecmp (name, vars[i].name))
@@ -1114,6 +1141,7 @@ xchat_list_get (xchat_plugin *ph, const char *name)
 		{
 			list->type = LIST_USERS;
 			list->head = list->next = userlist_flat_list (ph->context);
+			fe_userlist_set_selected (ph->context);
 			break;
 		}	/* fall through */
 
@@ -1179,7 +1207,7 @@ xchat_list_fields (xchat_plugin *ph, const char *name)
 	};
 	static const char * const users_fields[] =
 	{
-		"iaway", "shost", "tlasttalk", "snick", "sprefix", NULL
+		"iaway", "shost", "tlasttalk", "snick", "sprefix", "iselected", NULL
 	};
 	static const char * const list_of_lists[] =
 	{
@@ -1391,6 +1419,8 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		{
 		case 0x2de2ee:	/* away */
 			return ((struct User *)data)->away;
+		case 0x4705f29b: /* selected */
+			return ((struct User *)data)->selected;
 		}
 		break;
 
@@ -1425,10 +1455,11 @@ int
 xchat_emit_print (xchat_plugin *ph, const char *event_name, ...)
 {
 	va_list args;
+	/* currently only 4 because no events use more than 4.
+		This can be easily expanded without breaking the API. */
 	char *argv[4] = {NULL, NULL, NULL, NULL};
 	int i = 0;
 
-	memset (&argv, 0, sizeof (argv));
 	va_start (args, event_name);
 	while (1)
 	{
@@ -1466,7 +1497,7 @@ xchat_send_modes (xchat_plugin *ph, const char **targets, int ntargets, int mode
 char *
 xchat_strip (xchat_plugin *ph, const char *str, int len, int flags)
 {
-	return strip_color ((char *)str, len, (flags & 1), (flags & 2));
+	return strip_color ((char *)str, len, flags);
 }
 
 void
