@@ -56,6 +56,10 @@
 #include <gtk/gtkinvisible.h>
 #endif
 
+#ifdef USE_GTKSPELL
+#include <gtk/gtktextview.h>
+#endif
+
 #ifdef WIN32
 #include <windows.h>
 #endif
@@ -132,13 +136,13 @@ fe_args (int argc, char *argv[])
 	GOptionContext *context;
 
 #ifdef ENABLE_NLS
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	bind_textdomain_codeset (PACKAGE, "UTF-8");
-	textdomain (PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 #endif
 
 	context = g_option_context_new (NULL);
-	g_option_context_add_main_entries (context, gopt_entries, PACKAGE);
+	g_option_context_add_main_entries (context, gopt_entries, GETTEXT_PACKAGE);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_parse (context, &argc, &argv, &error);
 
@@ -153,7 +157,7 @@ fe_args (int argc, char *argv[])
 
 	if (arg_show_version)
 	{
-		printf (PACKAGE" "VERSION"\n");
+		printf (PACKAGE_NAME" "PACKAGE_VERSION"\n");
 		return 0;
 	}
 
@@ -501,12 +505,6 @@ fe_is_banwindow (struct session *sess)
    return 1;
 }
 
-void
-fe_chan_list_end (struct server *serv)
-{
-	gtk_widget_set_sensitive (serv->gui->chanlist_refresh, TRUE);
-}
-
 void
 fe_notify_update (char *name)
 {
@@ -582,20 +580,6 @@ fe_beep (void)
 	gdk_beep ();
 }
 
-typedef struct {
-	session *sess;
-	char *sstr;
-} fe_lastlog_info;
-
-static void
-fe_lastlog_foreach (GtkXText *xtext, unsigned char *text, void *data)
-{
-	fe_lastlog_info *info = data;
-
-	if (nocasestrstr (text, info->sstr))
-		PrintText (info->sess, text);
-}
-
 void
 fe_lastlog (session *sess, session *lastlog_sess, char *sstr)
 {
@@ -605,12 +589,7 @@ fe_lastlog (session *sess, session *lastlog_sess, char *sstr)
 	}
 	else
 	{
-		fe_lastlog_info info;
-
-		info.sess = lastlog_sess;
-		info.sstr = sstr;
-
-		gtk_xtext_foreach (sess->res->buffer, fe_lastlog_foreach, &info);
+		gtk_xtext_lastlog (lastlog_sess->res->buffer, sess->res->buffer, sstr);
 	}
 }
 
@@ -744,9 +723,24 @@ fe_ctrl_gui (session *sess, int action, int arg)
 	}
 }
 
+static void
+dcc_saveas_cb (struct DCC *dcc, char *file)
+{
+	if (file)
+		dcc_get_with_destfile (dcc, file);
+	else if (dcc->dccstat == STAT_QUEUED && dcc->resume_sent == 0)
+		dcc_abort (dcc->serv->front_session, dcc);
+}
+
 void
 fe_confirm (const char *message, void (*yesproc)(void *), void (*noproc)(void *), void *ud)
 {
+	/* warning, assuming fe_confirm is used by DCC only! */
+	struct DCC *dcc = ud;
+
+	if (dcc->file)
+		gtkutil_file_req (message, dcc_saveas_cb, ud, dcc->file,
+								FRF_WRITE|FRF_FILTERISINITIAL|FRF_NOASKOVERWRITE);
 }
 
 int
@@ -795,7 +789,7 @@ fe_get_inputbox_contents (session *sess)
 		return sess->res->input_text;
 
 	/* current focused tab */
-	return GTK_ENTRY (sess->gui->input_box)->text;
+	return SPELL_ENTRY_GET_TEXT (sess->gui->input_box);
 }
 
 int
@@ -806,7 +800,7 @@ fe_get_inputbox_cursor (session *sess)
 		return 0;
 
 	/* current focused tab */
-	return gtk_editable_get_position (GTK_EDITABLE (sess->gui->input_box));
+	return SPELL_ENTRY_GET_POS (sess->gui->input_box);
 }
 
 void
@@ -815,8 +809,8 @@ fe_set_inputbox_cursor (session *sess, int delta, int pos)
 	if (!sess->gui->is_tab || sess == current_tab)
 	{
 		if (delta)
-			pos += gtk_editable_get_position (GTK_EDITABLE (sess->gui->input_box));
-		gtk_editable_set_position (GTK_EDITABLE (sess->gui->input_box), pos);
+			pos += SPELL_ENTRY_GET_POS (sess->gui->input_box);
+		SPELL_ENTRY_SET_POS (sess->gui->input_box, pos);
 	} else
 	{
 		/* we don't support changing non-front tabs yet */
@@ -828,7 +822,7 @@ fe_set_inputbox_contents (session *sess, char *text)
 {
 	if (!sess->gui->is_tab || sess == current_tab)
 	{
-		gtk_entry_set_text (GTK_ENTRY (sess->gui->input_box), text);
+		SPELL_ENTRY_SET_TEXT (sess->gui->input_box, text);
 	} else
 	{
 		if (sess->res->input_text)
@@ -855,7 +849,7 @@ fe_open_url_locale (const char *url)
 		return;
 	}
 
-	moz = g_find_program_in_path ("gnome-moz-remote");
+	moz = g_find_program_in_path ("firefox");
 	if (moz)
 	{
 		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
